@@ -100,6 +100,7 @@ def train_from_folder(
     attn_res_layers = [32],
     freq_chan_attn = False,
     disc_output_size = 1,
+    dual_contrast_loss = False,
     antialias = False,
     interpolation_num_steps = 100,
     save_frames = False,
@@ -124,6 +125,7 @@ def train_from_folder(
         attn_res_layers = cast_list(attn_res_layers),
         freq_chan_attn = freq_chan_attn,
         disc_output_size = disc_output_size,
+        dual_contrast_loss = dual_contrast_loss,
         antialias = antialias,
         image_size = image_size,
         num_image_tiles = num_image_tiles,
